<?php

namespace GS_Instagram;

/**
 * This class is responsible for load admin specific funcitonality
 */
class Backend_Builder {

    /**
     * The single instance of the class.
     */
    protected static $_instance = null;

    /**
     * Initializes the Frontend_Builder() class
     *
     * Checks for an existing Frontend_Builder() instance and if it doesn't find one, creates it.
     */
    public static function instance() {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }

    /**
     * Constructor of Frontend_Builder class
     * Sets up all the appropriate hooks
     */
    private function __construct() {
        add_action( 'admin_menu', [ $this, 'admin_menu' ], 12 );

        add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_scripts' ], 10, 1 );

        add_action( 'wp_ajax_gs_instagram_admin_ajax', [ $this, 'load_ajax' ] );

        register_activation_hook( GS_INSTAGRAM__FILE__, [ $this, 'activation_hook' ] );

        $this->gs_libraries();
    }

    /**
     * Initialize amin menu
     */
    public function admin_menu() {
        require GS_INSTAGRAM_ADMIN_PATH . '/includes/Menu.php';

        new Backend\Menu();
    }

    /**
     * Initialize amin menu
     */
    public function enqueue_scripts( $page_hook ) {
        require GS_INSTAGRAM_ADMIN_PATH . '/includes/Enqueue.php';

        new Backend\Enqueue( $page_hook );
    }

    /**
     * handle ajax requests
     */
    public function load_ajax() {
        if ( ! isset( $_REQUEST['route'] ) ) {
            exit;
        }

        require GS_INSTAGRAM_ADMIN_PATH . '/includes/AjaxThemeHelper.php';
        require GS_INSTAGRAM_ADMIN_PATH . '/includes/Ajax.php';

        $ajax = new Backend\Ajax();

        if ( method_exists( $ajax , $_REQUEST['route'] ) ) {
            $args = isset( $_REQUEST['body'] ) ? $_REQUEST['body'] : [];

            call_user_func( [ $ajax, $_REQUEST['route'] ], $args );
        }

        exit;
    }

    /**
     * register_activation_hook
     */
    public function activation_hook() {
        require GS_INSTAGRAM_ADMIN_PATH . '/includes/Activation_Hook.php';

        new Backend\Activation_Hook();
    }

    /**
     * GS specific admin extensions
     */
    private function gs_libraries() {
        if ( ! class_exists( 'GSPlugins_wps' ) ) {
            require GS_INSTAGRAM_ADMIN_PATH . '/gs-lib/GSPlugins_wps.php';

            \GSPlugins_wps::get_instance();
        }
    }

}
