<?php

namespace GS_Instagram\Backend;

use Exception;
use GS_Instagram\Common\Instagram_Client;

class Ajax {

    use AjaxThemeHelper;

    /**
     * save settings values
     */
    public function save_settings( $data ) {
        update_option( 'gs_instagram_custom_css', strval($data['custom_css']), false );

        unset( $data['custom_css'] );

        update_option( 'gs_instagram_settings', $data, false );

        wp_send_json_success();
    }

    /**
     * get settings values
     */
    public function get_settings() {
        $data = get_option( 'gs_instagram_settings', [] );

        $data['custom_css'] = (string) get_option( 'gs_instagram_custom_css', '' );

        wp_send_json_success( $data );
    }

    /**
     * Get theme create preview data
     */
    public function get_preview_data( $data ) {
        $client    = new Instagram_Client();

        try {
            if ( $data['source_type'] == 'user' ) {
                $meta_data = $client->scrap_user_meta_data( $data['source'] );
                $feeds     = $client->fetch( $meta_data['user_id'], $data['per_page'], $data['source_type'] );
            }

            if ( $data['source_type'] == 'tag' ) {
                $feeds = $client->fetch( $data['source'], $data['per_page'], $data['source_type'] );
            }
        } catch ( Exception $e ) {
            wp_send_json_error( [
                'message' => $e->getMessage()
            ], 422 );
        }

        wp_send_json_success( [
            'edges'          => array_slice( $feeds['edges'], 0, $data['per_page'] ),
            'user_meta_data' => isset( $meta_data, $meta_data['user_id'] ) ? $meta_data : null,
        ] );
    }

    /**
     * get themes list
     */
    public function get_themes() {
        global $wpdb;
        $table_name = $wpdb->prefix . 'gs_instagram_themes';

        $data = $wpdb->get_results( "SELECT * FROM {$table_name} ORDER BY `id` DESC;", ARRAY_A );

        wp_send_json_success( $data );
    }

    /**
     * get sigle theme
     */
    public function get_theme( $data ) {
        global $wpdb;
        $table_name = $wpdb->prefix . 'gs_instagram_themes';
        $id = intval( $data['id'] );

        $data = $wpdb->get_row( "SELECT * FROM {$table_name} WHERE `id` = {$id} LIMIT 1;", ARRAY_A );

        wp_send_json_success( $data );
    }

}
