<?php

namespace GS_Instagram\Common;

use Exception;

class Instagram_Model {

    /**
     * Get themes data
     */
    public static function get_theme_posts( $data, $page = 1, $transient ) {
        $_this = new self();

        return $_this->get_posts( $data, $page, $transient );
    }

    /**
     * Get themes data
     */
    private function get_posts( $data, $page, $transient ) {
        $results = get_transient( $transient );

        $start = ( $page - 1 ) * $data['per_page'];

        // if has data in transient
        if ( false !== $results && ! empty( $results['edges'] ) ) {
            $edges = array_slice( $results['edges'], $start, $data['per_page'] );

            // If we don't have any cache edges and API has more edges
            if ( count( $edges ) < $data['per_page'] && $results['page_info']['has_next_page'] ) {
                $feeds = $this->fetch_instagram_feeds( $data, $results['page_info']['end_cursor'] );
                $feeds['edges'] = array_merge( $results['edges'], $feeds['edges'] );
            } else {
                return [
                    'items'         => $edges,
                    'has_next_page' => $results['page_info']['has_next_page']
                ];
            }
        } else {
            // If no data in transient then fetch data and cache it
            $feeds = $this->fetch_instagram_feeds( $data );

            if ( $page != 1 ) {
                error_log( 'TODO: Need to manage this condition - ' . $page . ' - ' . $data['id'] );
            }
        }

        if ( empty( $feeds['edges'] ) ) {
            throw new Exception( 'No instagram feeds found.', 110 );
        }

        // Cache feeds
        set_transient( $transient, $feeds, $this->get_transient_expiration() );

        $edges = array_slice( $feeds['edges'], $start, $data['per_page'] );

        return [
            'items'         => $edges,
            'has_next_page' => $feeds['page_info']['has_next_page']
        ];
    }


    /**
     * Fetch data from API
     */
    private function fetch_instagram_feeds( $data, $end_cursor = '' ) {
        $client = new Instagram_Client();

        if ( $data['source_type'] == 'user' ) {
            $meta_data = $client->scrap_user_meta_data( $data['source'] );
            return $client->fetch( $meta_data['user_id'], $data['per_page'], $data['source_type'], $end_cursor );
        }

        if ( $data['source_type'] == 'tag' ) {
            return $client->fetch( $data['source'], $data['per_page'], $data['source_type'], $end_cursor );
        }
    }

    /**
     * Get cache time
     */
    private function get_transient_expiration() {
        $settings = get_option( 'gs_instagram_settings', [] );

        if ( ! isset( $settings['cache_time'] ) ) {
            return HOUR_IN_SECONDS;
        }

        $cache_time = intval( $settings['cache_time'] );

        if ( $cache_time < 10 ) {
            return 10 * 60;
        }

        return $cache_time * 60;
    }

}
