<?php

namespace GS_Instagram\Frontend;

use Exception;
use GS_Instagram\Common\Instagram_Model;
use GS_Instagram\Common\Instagram_Client;

class Ajax {

    /**
     * Handle first request from frontend
     * Response with theme information and instagram posts
     */
    public function get_initial_posts( $body ) {
        if ( empty( $body['themeid'] ) ) {
            $this->send_error_message( 'Incorrect shortcode, Please use correct shortcode with id.' );
        }

        $theme_data = $this->get_shortcode_theme( $body['themeid'] );
        $posts      = $this->get_theme_posts( $theme_data );

        $posts['theme'] = $theme_data;

        wp_send_json_success( $posts );
    }

    /**
     * Get themes data
     */
    private function get_theme_posts( $data, $page = 1 ) {
        $transient = 'gs_theme_' . $data['id'] . '_posts';

        try {

            return Instagram_Model::get_theme_posts( $data, $page, $transient );

        } catch( Exception $e ) {
            $this->send_error_message( $e->getMessage() );
        }
    }

    /**
     * Response pagination wise posts
     */
    public function get_pagination_posts( $body ) {
        $theme_data = $this->get_shortcode_theme( $body['themeid'] );

        $posts = $this->get_theme_posts( $theme_data, $body['page'] );

        wp_send_json_success( $posts );
    }

    /**
     * Get theme of shortcode
     */
    private function get_shortcode_theme( $themeid ) {
        $themeid = intval( $themeid );

        global $wpdb;
        $table_name = $wpdb->prefix . 'gs_instagram_themes';

        $theme_data = $wpdb->get_row( "
            SELECT * FROM {$table_name} WHERE `id` = {$themeid};
        ", ARRAY_A );

        if ( empty( $theme_data ) ) {
            $this->send_error_message( 'Incorrect shortcode id, Please use correct shortcode with id.' );
        }

        return $theme_data;
    }

    /**
     * Send error message
     */
    private function send_error_message( $message = null ) {
        wp_send_json_error( [
            'message' => $message
        ], 422 );
    }

}
